#!/System/Library/Frameworks/Ruby.framework/Versions/1.8/usr/bin/ruby

require 'osx/cocoa'
include OSX

libraryDirectoryPath = File.dirname(__FILE__)
$LOAD_PATH << libraryDirectoryPath

require 'xcarchive.rb'
require 'codesign.rb'
require 'bundle.rb'

def usage
	print "Usage:\n"
	print "-applicationPath: path to the application bundle to be passed to productbuild [required]\n"
	print "-packagePath: path to write the Product.pkg file to [required]\n"
	print "\n"
	print "Example:\n"
	print "ruby application_productbuild.rb\n -applicationPath ~/Projects/Build\\ Archive/Courier/courier_4770_1.2.1rc1-appstore.zip -productPath ~/Desktop/Courier.pkg\n"
	print "\n"
	exit 1
end

if ARGV.count == 0 then
	usage
end

def start
	print "============    PRODUCTBUILD STARTED    ============\n"
	print "\n"
end

def fail(reason)
	print "\n"
	print "============    PRODUCTBUILD FAILURE    ============\n"
	print "\n"
	print "error: #{reason}\n"
	print "\n"
	exit 1
end

def finish
	print "\n"
	print "============    PRODUCTBUILD SUCCESS    ============\n"
end


# Check arguments

applicationPath = NSUserDefaults.standardUserDefaults.stringForKey "applicationPath"
if applicationPath.nil?
	fail "an application path must be specified using -applicationPath"
end
unless NSFileManager.defaultManager.fileExistsAtPath applicationPath
	fail "application does not exist at the specified path"
end
applicationLocation = NSURL.fileURLWithPath applicationPath

packagePath = NSUserDefaults.standardUserDefaults.objectForKey "packagePath"
if packagePath.nil?
	fail "a package path must be specified using -packagePath"
	exit 1
end
if NSFileManager.defaultManager.fileExistsAtPath packagePath
	fail "a package already exists at the specified packagePath"
	exit 1
end


#

start


# If passed a ZIP, get a path to the actual application

applicationLocationTypeAndError = applicationLocation.getResourceValue_forKey_error NSURLTypeIdentifierKey
unless applicationLocationTypeAndError[0]
	fail "couldn't determine type of application"
end
applicationLocationType = applicationLocationTypeAndError[1]

if applicationLocationType.isEqualToString "public.zip-archive"
	temporaryDirectoryLocation = NSURL.fileURLWithPathComponents [ NSTemporaryDirectory(), "com.realmacsoftware.dts.productbuild", NSProcessInfo.processInfo.globallyUniqueString ].to_ns
	
	unarchiveTask = NSTask.launchedTaskWithLaunchPath_arguments_ "/usr/bin/ditto", [ "-x", "-k", applicationLocation.path, temporaryDirectoryLocation.path ].to_ns
	unarchiveTask.waitUntilExit
	
	fail "unarchive failed" unless unarchiveTask.terminationStatus == 0
	
	temporaryDirectoryContentsAndError = NSFileManager.defaultManager.contentsOfDirectoryAtURL_includingPropertiesForKeys_options_error_ temporaryDirectoryLocation, NSArray.array, 0
	fail "couldn't get contents of extracted folder" unless temporaryDirectoryContentsAndError[0]
	
	temporaryDirectoryContents = temporaryDirectoryContentsAndError[0]
	fail "archive contains more than one file" unless temporaryDirectoryContents.count == 1
	
	newApplicationLocation = temporaryDirectoryContents.pathsMatchingExtensions([ "app" ].to_ns).lastObject
	fail "couldn't find application in extracted folder" if newApplicationLocation.nil?
	
	applicationLocation = newApplicationLocation
elsif applicationLocationType.isEqualToString "com.apple.application-bundle"
	#nop
else
	fail "unknown application type #{applicationLocationType}"
end


# Generate the package

print "Invoking productbuild\n"

$stdout.flush
productbuildTask = NSTask.launchedTaskWithLaunchPath_arguments_ "/usr/bin/productbuild", [ "--component", applicationLocation.path, "/Applications", "--sign", "3rd Party Mac Developer Installer: Realmac Software Limited", packagePath ].to_ns
productbuildTask.waitUntilExit

fail "productbuild failed" unless productbuildTask.terminationStatus == 0

# Open Application Loader

openApplicationLoaderTask = NSTask.launchedTaskWithLaunchPath_arguments_ "/usr/bin/open", [ "-a", "Application Loader" ].to_ns
openApplicationLoaderTask.waitUntilExit


#

finish
